@@ -59,6 +59,7 @@ class SCAnalysis(BaseAnalysisElement):
         self.dimension_adaptive = sampler.dimension_adaptive
         if self.dimension_adaptive:
             self.adaptation_errors = []
+            self.mean_history = []
         self.sparse = sampler.sparse
 
     def element_name(self):
@@ -84,7 +85,7 @@ class SCAnalysis(BaseAnalysisElement):
 
         """
         print("Saving analysis state to %s" % filename)
-        #make a copy of the state, and do not store the sampler as well
+        # make a copy of the state, and do not store the sampler as well
         state = copy.copy(self.__dict__)
         del state['sampler']
         file = open(filename, 'wb')
@@ -137,7 +138,7 @@ class SCAnalysis(BaseAnalysisElement):
 
         # the number of uncertain parameters
         self.N = self.sampler.N
-        #tensor grid
+        # tensor grid
         self.xi_d = self.sampler.xi_d
         # the maximum level (quad order) of the (sparse) grid
         self.L = self.sampler.L
@@ -151,10 +152,10 @@ class SCAnalysis(BaseAnalysisElement):
         # For sparse grid: one or more levels
         else:
             self.L_min = 1
-            #multi indices (stored in l_norm) for isotropic sparse grid or
-            #dimension-adaptive grid before the 1st refinement.
-            #If dimension_adaptive and number_of_adaptations > 0: l_norm
-            #is computed in self.adaptation_metric
+            # multi indices (stored in l_norm) for isotropic sparse grid or
+            # dimension-adaptive grid before the 1st refinement.
+            # If dimension_adaptive and number_of_adaptations > 0: l_norm
+            # is computed in self.adaptation_metric
             if not self.dimension_adaptive or self.sampler.number_of_adaptations == 0:
                 # the maximum level (quad order) of the (sparse) grid
                 self.l_norm = self.sampler.compute_sparse_multi_idx(self.L, self.N)
@@ -259,7 +260,7 @@ class SCAnalysis(BaseAnalysisElement):
         logging.debug('done.')
         return map_
 
-    def adapt_dimension(self, qoi, data_frame):
+    def adapt_dimension(self, qoi, data_frame, store_mean_history=False):
         """
         Compute the adaptation metric and decide which of the admissible
         level indices to include in next iteration of the sparse grid. The
@@ -280,48 +281,53 @@ class SCAnalysis(BaseAnalysisElement):
         None.
 
         """
-        #load the code samples
+        # load the code samples
         samples = []
         for run_id in data_frame.run_id.unique():
             values = data_frame.loc[data_frame['run_id'] == run_id][qoi].values
             samples.append(values)
 
-        #the currently accepted grid points
+        # the currently accepted grid points
         xi_d_accepted = self.sampler.generate_grid(self.l_norm)
 
-        #compute the hierarchical surplus based error for every admissible l
+        # compute the hierarchical surplus based error for every admissible l
         error = {}
         for l in self.sampler.admissible_idx:
             error[tuple(l)] = []
             # collocation points of current level index l
             X_l = [self.sampler.xi_1d[n][l[n]] for n in range(self.N)]
             X_l = np.array(list(product(*X_l)))
-            #only consider new points, subtract the accepted points
+            # only consider new points, subtract the accepted points
             X_l = setdiff2d(X_l, xi_d_accepted)
             for xi in X_l:
-                #find the location of the current xi in the global grid
+                # find the location of the current xi in the global grid
                 idx = np.where((xi == self.sampler.xi_d).all(axis=1))[0][0]
-                #hierarchical surplus error at xi
+                # hierarchical surplus error at xi
                 hier_surplus = samples[idx] - self.surrogate(qoi, xi)
-                error[tuple(l)].append(np.linalg.norm(hier_surplus))
-            #compute mean error over all points in X_l
+                error[tuple(l)].append(np.linalg.norm(hier_surplus, np.inf))
+            # compute mean error over all points in X_l
             error[tuple(l)] = np.mean(error[tuple(l)])
         for key in error.keys():
             print("Surplus error when l =", key, "=", error[key])
-        #find the admissble index with the largest error
+        # find the admissble index with the largest error
         l_star = np.array(max(error, key=error.get)).reshape([1, self.N])
         print('Selecting', l_star, 'for refinement.')
-        #add max error to list
+        # add max error to list
         self.adaptation_errors.append(max(error.values()))
 
-        #add l_star to the current accepted level indices
+        # add l_star to the current accepted level indices
         self.l_norm = np.concatenate((self.l_norm, l_star))
-        #if someone executes this function twice for some reason,
-        #remove the duplicate l_star entry
-        self.l_norm = np.unique(self.l_norm, axis=0)
+        # if someone executes this function twice for some reason,
+        # remove the duplicate l_star entry. Keep order unaltered
+        idx = np.unique(self.l_norm, axis=0, return_index=True)[1]
+        self.l_norm = self.l_norm[idx]
 
+        # peform the analyse step, but do not compute moments and Sobols
         self.analyse(data_frame, compute_results=False)
 
+        if store_mean_history:
+            self.mean_history.append(self.quadrature(qoi))
+
         # self.L = np.max(np.sum(self.l_norm, axis = 1) - self.N + 1)
         # self.xi_d = self.sampler.generate_grid(self.l_norm)
         # self.run_id = []
@@ -344,6 +350,37 @@ class SCAnalysis(BaseAnalysisElement):
         """
         return self.adaptation_errors
 
+    def plot_mean_convergence(self):
+        """
+        Plots the convergence of the statistical mean over the different
+        refinements in a dimension-adaptive setting.
+
+        Returns
+        -------
+        None.
+
+        """
+        if not self.dimension_adaptive:
+            print('Only works for the dimension adaptive sampler.')
+            return
+
+        K = len(self.mean_history)
+        if K < 2:
+            print('Means from at least two refinements are required')
+            return
+        else:
+            differ = np.zeros(K - 1)
+            for i in range(1, K):
+                differ[i - 1] = np.linalg.norm(self.mean_history[i] -
+                                               self.mean_history[i - 1], np.inf)
+        import matplotlib.pyplot as plt
+        fig = plt.figure(figsize=[4, 4])
+        ax = fig.add_subplot(111, xlabel=r'refinement step',
+                             ylabel=r'$ ||\mathrm{mean}_i - \mathrm{mean}_{i - 1}||_\infty$')
+        ax.plot(range(2, K + 1), differ, '-b+')
+        plt.tight_layout()
+        plt.show()
+
     def surrogate(self, qoi, x, L=None):
         """
         Use sc_expansion UQP as a surrogate
@@ -405,14 +442,14 @@ class SCAnalysis(BaseAnalysisElement):
         if samples is None:
             samples = self.samples[qoi]
 
-        #quadrature Q
+        # quadrature Q
         Q = 0.0
 
-        #loop over l
+        # loop over l
         for l in self.l_norm:
 
-            #for sum(l) < L, combination technique formula shows that weights
-            #are zero
+            # for sum(l) < L, combination technique formula shows that weights
+            # are zero
             if np.sum(l) >= self.L:
 
                 # compute the tensor product of parameter and weight values
@@ -424,7 +461,7 @@ class SCAnalysis(BaseAnalysisElement):
                 W_k = np.prod(W_k, axis=1)
                 W_k = W_k.reshape([W_k.shape[0], 1])
 
-                #scaling factor of combination technique
+                # scaling factor of combination technique
                 scaling_factor = (-1)**(self.L + self.N - np.sum(l) - 1) * \
                     comb(self.N - 1, np.sum(l) - self.L)
                 W_k *= scaling_factor
@@ -566,11 +603,11 @@ class SCAnalysis(BaseAnalysisElement):
             # Recursively computed.
 
             if self.sparse:
-                #In case of sparse grid: level = sum(l) - N + 1
-                #so prev level is sum(l) - N
+                # In case of sparse grid: level = sum(l) - N + 1
+                # so prev level is sum(l) - N
                 Lm1 = np.sum(l) - self.N
             else:
-                #previous level of full tensor grid = L - 1, will yield a zero
+                # previous level of full tensor grid = L - 1, will yield a zero
                 Lm1 = self.L - 1
 
             if k in self.surr_lm1[L]:
@@ -708,7 +745,7 @@ class SCAnalysis(BaseAnalysisElement):
         ax = fig.add_subplot(111, ylabel='max quadrature order',
                              title='Number of refinements = %d'
                              % self.sampler.number_of_adaptations)
-        #find max quad order for every parameter
+        # find max quad order for every parameter
         adapt_measure = np.max(self.l_norm, axis=0)
         ax.bar(range(adapt_measure.size), height=adapt_measure)
         params = list(self.sampler.vary.get_keys())
