@@ -12,6 +12,7 @@ from easyvvuq import ParamsSpecification
 from easyvvuq.constants import default_campaign_prefix, Status
 from easyvvuq.data_structs import RunInfo, CampaignInfo, AppInfo
 from easyvvuq.sampling import BaseSamplingElement
+from easyvvuq.actions import ActionStatuses
 
 __copyright__ = """
 
@@ -620,7 +621,7 @@ class Campaign:
         for run_id, run_data in self.campaign_db.runs(status=status, app_id=self._active_app['id']):
             fn(run_id, run_data)
 
-    def apply_for_each_run_dir(self, action, status=Status.ENCODED):
+    def apply_for_each_run_dir(self, action, status=Status.ENCODED, batch_size=1):
         """
         For each run in this Campaign's run list, apply the specified action
         (an object of type Action)
@@ -630,16 +631,44 @@ class Campaign:
         action : the action to be applied to each run directory
             The function to be applied to each run directory. func() will
             be called with the run directory path as its only argument.
+        status : Status
+            Will apply the action only to those runs whose status is as specified
 
         Returns
         -------
+        action_statuses: ActionStatuses
+            An object containing ActionStatus instances to track action execution
         """
 
         # Loop through all runs in this campaign with status ENCODED, and
         # run the specified action on each run's dir
+        action_statuses = []
         for run_id, run_data in self.campaign_db.runs(status=status, app_id=self._active_app['id']):
-            logger.info("Applying " + action.__module__ + " to " + run_data['run_dir'])
-            action.act_on_dir(run_data['run_dir'])
+            action_statuses.append(action.act_on_dir(run_data['run_dir']))
+        return ActionStatuses(action_statuses, batch_size=batch_size)
+
+    def sample_and_apply(self, nsamples, action, batch_size):
+        """This will draw samples, populated the runs directories and run the specified action.
+        This is a convenience method.
+
+        Parameters
+        ----------
+        nsamples : int
+            number of samples to draw
+        action : BaseAction
+            an action to be executed
+        batch_size : int
+            number of actions to be executed at the same time
+
+        Returns
+        -------
+        action_statuses: ActionStatuses
+            An object containing ActionStatus instances to track action execution
+        """
+        self.draw_samples(nsamples)
+        self.populate_runs_dir()
+        action_statuses = self.apply_for_each_run_dir(action, batch_size=batch_size)
+        return action_statuses
 
     def collate(self):
         """Combine the output from all runs associated with the current app.
